#include "streetsmodel.h"

StreetsModel::StreetsModel(QObject *parent) :
    QAbstractItemModel(parent)
{
    m_cityesModel = new QSqlRelationalTableModel();
    m_cityesModel->setTable("cityes");
    m_cityesModel->select();

    m_streetsModel = new QSqlRelationalTableModel();
    m_streetsModel->setTable("streets");
    m_streetsModel->setRelation(3,QSqlRelation("cityes","id","name"));
    m_streetsModel->select();

    m_organizationsModel = new QSqlRelationalTableModel();
    m_organizationsModel->setTable("organizations");
    m_organizationsModel->setRelation(2,QSqlRelation("streets","id","name"));
    m_organizationsModel->select();
}

void StreetsModel::setCityesFilter( const QString &cityId ) const
{
    QString filter = "cityid = " + cityId;
    if( m_streetsModel->filter() != filter )
        m_streetsModel->setFilter(filter);
}

void StreetsModel::setStreetFilter( const QString &streetId ) const
{
    QString filter = "streetid = " + streetId;
    if( m_organizationsModel->filter() != filter )
        m_organizationsModel->setFilter(filter);

}

int StreetsModel::columnCount ( const QModelIndex & parent ) const
{
    if( !parent.isValid() )
    {
        return m_cityesModel->columnCount();
    }

    InternalId *id = (InternalId*)parent.internalId();
    if( id->table == CityesTable )
        return m_streetsModel->columnCount();

    if( id->table == StreetsTable )
        return m_organizationsModel->columnCount();

    throw "WTF?";
}

QVariant StreetsModel::data ( const QModelIndex & index, int role ) const
{
    InternalId *id = (InternalId*)index.internalId();
    if( id->table == CityesTable )
        return m_cityesModel->index( index.row(), index.column() ).data( role );

    if( id->table == StreetsTable )
    {
        setCityesFilter(id->parentIndex.data(Qt::EditRole).toString());
        return m_streetsModel->index( index.row(), index.column() ).data( role );
    }

    if( id->table == OrganizationsTable )
    {
        setStreetFilter(id->parentIndex.data(Qt::EditRole).toString());
        return m_organizationsModel->index( index.row(), index.column() ).data( role );
    }

    throw "WTF?";
}

QModelIndex StreetsModel::index ( int row, int column, const QModelIndex & parent ) const
{
    InternalId internalId;
    internalId.row = row;
    internalId.column = column;

    if( !parent.isValid() )
    {
        internalId.table = CityesTable;
        internalId.parentIndex = QModelIndex();
    }
    else
    {
        InternalId *parentInternalId = (InternalId*)parent.internalId();
        if( parentInternalId->table == CityesTable )
        {
            internalId.table = StreetsTable;
            internalId.parentIndex = index( parent.row(), 0, parentInternalId->parentIndex );
        }
        else if( parentInternalId->table == StreetsTable )
        {
            internalId.table = OrganizationsTable;
            internalId.parentIndex = index( parent.row(), 0, parentInternalId->parentIndex );
        }
        else
        {
            return QModelIndex();
        }
    }

    InternalId *resultInternalId = 0;
    for( int i = 0; i < m_internalIds.count(); i++ )
    {
        if( m_internalIds.at(i) == internalId )
        {
            resultInternalId = &m_internalIds[i];
            break;
        }
    }

    if( resultInternalId == 0 )
    {
        m_internalIds << internalId;
        resultInternalId = &m_internalIds[ m_internalIds.count()-1 ];
    }

    return createIndex(row,column,(void*)resultInternalId);
}

QModelIndex StreetsModel::parent ( const QModelIndex & index ) const
{
    InternalId *id = (InternalId*)index.internalId();
    return id->parentIndex;
}

int StreetsModel::rowCount ( const QModelIndex & parent ) const
{
    if( !parent.isValid() )
        return m_cityesModel->rowCount();

    InternalId *id = (InternalId*)parent.internalId();
    if( id->table == CityesTable )
    {
        setCityesFilter(parent.data(Qt::EditRole).toString());
        return m_streetsModel->rowCount();
    }

    if( id->table == StreetsTable )
    {
        setStreetFilter(parent.data(Qt::EditRole).toString());
        return m_organizationsModel->rowCount();
    }

    return 0;
}
